﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class Engineer : Part
    {
        Ship ship = new Ship();
        ThrustToWeight ttw = new ThrustToWeight();
        Rect winPos = new Rect(300, 70, 0, 0);
        GUIStyle winStyle, areaStyle, titleStyle, headStyle, dataStyle, buttonStyle;
        bool hasInitGUI = false;
        bool drawStandard = true;
        bool drawThrust = false;
        bool drawAllStages = false;

        public bool IsPrimary
        {
            get;
            set;
        }

        public string Version
        {
            get { return "0.3"; }
        }

        protected override void onPartAttach(Part parent)
        {
            this.manualStageOffset = 0;

            if (CheckPrimary())
            {
                IsPrimary = true;
                PopulateStages(EditorLogic.SortedShipList);
                RenderingManager.AddToPostDrawQueue(0, DrawGUI);
            }
            else
            {
                IsPrimary = false;
            }

            base.onPartAttach(parent);
        }

        protected override void onPartDetach()
        {
            RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);
            base.onPartDetach();
        }

        protected override void onEditorUpdate()
        {
            if (CheckPrimary())
            {
                PopulateStages(EditorLogic.SortedShipList);
            }
            else
            {
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);
            }
  
            base.onEditorUpdate();
        }

        public bool CheckPrimary()
        {
            foreach (Part part in EditorLogic.SortedShipList)
            {
                if (part != this && part is Engineer)
                {
                    if ((part as Engineer).IsPrimary)
                    {
                        return false;
                    }
                }
            }

            return true;
        }

        public Stage GetStage(int number)
        {
            try
            {
                return ship.Stages[number];
            }
            catch
            {
                return ship.Stages[number - 1];
            }
        }

        public void PopulateStages(List<Part> parts)
        {
            int stageCount = ship.Stages.Count;
            ship.Update(parts);
            if (stageCount > ship.Stages.Count)
                winPos.height = 0;
        }

        public void DrawGUI()
        {
            if (!hasInitGUI)
                InitGUI();

            winPos = GUILayout.Window(3, winPos, MainWindow, "Kerbal Engineer Redux -  Version " + Version, winStyle);
        }

        public void MainWindow(int windowId)
        {
            GUILayout.BeginHorizontal();
            string stdButtonText = "Hide Standard Specification";
            if (!drawStandard)
                stdButtonText = "Show Standard Specification";

            string ttwButtonText = "Hide Thrust to Weight Ratios";
            if (!drawThrust)
                ttwButtonText = "Show Thrust to Weight Ratios";

            if (GUILayout.Button(stdButtonText, buttonStyle))
            {
                drawStandard = !drawStandard;
                winPos.height = 0;
            }

            if (GUILayout.Button(ttwButtonText, buttonStyle))
            {
                drawThrust = !drawThrust;
                winPos.height = 0;
            }
            GUILayout.EndHorizontal();

            if (drawStandard)
                DrawStandard();

            if (drawThrust)
                DrawThrust();

            GUI.DragWindow(new Rect(0, 0, winPos.width, winPos.height));
        }

        private void DrawStandard()
        {
            GUILayout.BeginHorizontal();
            GUILayout.Label("Standard Specifications", titleStyle);
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);

            GUILayout.BeginVertical();
            GUILayout.Label("", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful && !drawAllStages)
                    GUILayout.Label("S " + Convert.ToString(i), headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Mass", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].Mass.ToString("0.00") + " / " + ship.Stages[i].TotalMass.ToString("0.00"), dataStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Fuel", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].Fuel.ToString("0"), dataStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Isp(A/V)", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].AtmIsp.ToString("0") + " / " + ship.Stages[i].VacIsp.ToString("0"), dataStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Thrust", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].Thrust.ToString("0"), dataStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label("Delta-V", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label(ship.Stages[i].DeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.EndVertical();

            GUILayout.EndHorizontal();
        }

        private void DrawThrust()
        {
            GUILayout.BeginHorizontal();
            GUILayout.Label("Thrust to Weight Ratios", titleStyle);
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);

            GUILayout.BeginVertical();
            GUILayout.Label("", headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful && !drawAllStages)
                    GUILayout.Label("S " + Convert.ToString(i), headStyle);
            GUILayout.EndVertical();

            foreach (string influenceName in ttw.GravityList.Keys.ToArray<string>())
            {
            GUILayout.BeginVertical();
            GUILayout.Label(influenceName, headStyle);
            for (int i = 0; i < ship.Stages.Count; i++)
                if (ship.Stages[i].Useful || drawAllStages)
                    GUILayout.Label((ship.Stages[i].Thrust / (ship.Stages[i].TotalMass * ttw.GravityList[influenceName])).ToString("0.00"), dataStyle);
            GUILayout.EndVertical();
            }

            GUILayout.EndHorizontal();
        }

        private void InitGUI()
        {
            GUI.skin = HighLogic.Skin;

            if (winStyle == null)
            {
                winStyle = new GUIStyle(GUI.skin.window);
                winStyle.fixedWidth = 700;
                winStyle.stretchHeight = true;
            }

            if (areaStyle == null)
            {
                areaStyle = new GUIStyle(GUI.skin.textArea);
                areaStyle.hover = areaStyle.active = areaStyle.normal;
            }

            if (titleStyle == null)
            {
                titleStyle = new GUIStyle(GUI.skin.label);
                titleStyle.normal.textColor = Color.yellow;
                titleStyle.fontStyle = FontStyle.Bold;
            }

            if (headStyle == null)
            {
                headStyle = new GUIStyle(GUI.skin.label);
                headStyle.normal.textColor = Color.white;
                headStyle.fontStyle = FontStyle.Bold;
            }

            if (dataStyle == null)
            {
                dataStyle = new GUIStyle(GUI.skin.label);
            }

            if (buttonStyle == null)
            {
                buttonStyle = new GUIStyle(GUI.skin.button);
            }

            hasInitGUI = true;
        }
    }
}
