﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class FlightEngineer : PartModule
    {
        Rect winPos;
        GUIStyle winStyle, areaStyle, buttonStyle, titleStyle, headStyle, dataStyle;
        bool hasInitGUI = false;
        bool showDisplay = true;

        public override void OnStart(PartModule.StartState state)
        {
            if (state == StartState.Editor)
            {
                foreach (Part part in EditorLogic.SortedShipList)
                {
                    if (part is Engineer)
                    {
                        this.part.onAttach(this.part.parent);
                        return;
                    }
                }
            }

            if (state != StartState.Editor)
            {
                this.part.mass = 1.0E-006f;

                foreach (Part part in vessel.parts)
                {
                    if (part != this.part && part is Engineer)
                    {
                        if ((part as Engineer).IsPrimary)
                        {
                            (this.part as Engineer).IsPrimary = false;
                            return;
                        }
                    }
                }

                (this.part as Engineer).IsPrimary = true;

                RenderingManager.AddToPostDrawQueue(0, DrawGUI);
            }

            base.OnStart(state);
        }

        public void DrawGUI()
        {
            if (!hasInitGUI)
                InitGUI();

            (this.part as Engineer).PopulateStages(vessel.parts);

            winPos = GUILayout.Window(3, winPos, MainWindow, "Flight Engineer", winStyle);
        }

        public void MainWindow(int windowId)
        {
            string btnText = "Show";
            if (showDisplay)
                btnText = "Hide";

            GUILayout.BeginHorizontal();
            if (GUILayout.Button(btnText, buttonStyle))
            {
                showDisplay = !showDisplay;
                winPos.height = 0;
            }
            GUILayout.EndHorizontal();

            if (showDisplay)
                DrawDisplay();

            GUI.DragWindow(new Rect(0, 0, winPos.width, winPos.height));
        }

        private void DrawDisplay()
        {
            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Altitude (True)", headStyle);
            GUILayout.Label("Apoapsis Height", headStyle);
            GUILayout.Label("Time to Apoapsis", headStyle);
            GUILayout.Label("Periapsis Height", headStyle);
            GUILayout.Label("Time to Periapsis", headStyle);
            GUILayout.Label("Inclination", headStyle);
            GUILayout.Label("Eccentricity", headStyle);
            GUILayout.Label("Orbital Period", headStyle);
            GUILayout.Label("Semi-major Axis", headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(FormatDistance(vessel.mainBody.GetAltitude(vessel.CoM) - vessel.terrainAltitude), dataStyle);
            GUILayout.Label(FormatDistance(vessel.orbit.ApA), dataStyle);
            GUILayout.Label(vessel.orbit.timeToAp.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(FormatDistance(vessel.orbit.PeA), dataStyle);
            GUILayout.Label(vessel.orbit.timeToPe.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(vessel.orbit.inclination.ToString("0.000") + "°", dataStyle);
            GUILayout.Label(vessel.orbit.eccentricity.ToString("0.000000"), dataStyle);
            GUILayout.Label(vessel.orbit.period.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(FormatDistance(vessel.orbit.semiMajorAxis), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Delta-V (Stage)", headStyle);
            GUILayout.Label("Delta-V (Total)", headStyle);
            GUILayout.Label("Fuel", headStyle);
            GUILayout.Label("Mass", headStyle);
            GUILayout.Label("TWR", headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label((this.part as Engineer).GetStage(vessel.currentStage).DeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.Label((this.part as Engineer).GetStage(vessel.currentStage).TotalDeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.Label((this.part as Engineer).GetStage(vessel.currentStage).Fuel.ToString("0.00") + " / " + (this.part as Engineer).GetStage(vessel.currentStage).TotalFuel.ToString("0.00"), dataStyle);
            GUILayout.Label((this.part as Engineer).GetStage(vessel.currentStage).Mass.ToString("0.00") + " / " + (this.part as Engineer).GetStage(vessel.currentStage).TotalMass.ToString("0.00"), dataStyle);
            GUILayout.Label(((this.part as Engineer).GetStage(vessel.currentStage).Thrust / ((this.part as Engineer).GetStage(vessel.currentStage).TotalMass * (vessel.mainBody.gravParameter / Math.Pow(vessel.mainBody.Radius, 2)))).ToString("0.00"), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Vertical Speed", headStyle);
            GUILayout.Label("Horizontal Speed", headStyle);
            GUILayout.Label("G-Force", headStyle);
            GUILayout.Label("Longitude", headStyle);
            GUILayout.Label("Latitude", headStyle);
            GUILayout.Label("Atmospheric Density", headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(FormatSpeed(vessel.verticalSpeed), dataStyle);
            GUILayout.Label(FormatSpeed(vessel.horizontalSrfSpeed), dataStyle);
            GUILayout.Label(vessel.geeForce.ToString("0.000"), dataStyle);

            GUILayout.Label(vessel.longitude.ToString("0.000000") + "°", dataStyle);
            GUILayout.Label(vessel.latitude.ToString("0.000000") + "°", dataStyle);

            GUILayout.Label(vessel.atmDensity.ToString("0.000000"), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        private string FormatSpeed(double number)
        {
            bool negative = false;
            if (number < 0)
            {
                number = -number;
                negative = true;
            }

            string notation = "m/s";
            string strNumber = number.ToString("#,##0.0");

            if (number < 1)
            {
                number *= 100;
                if (number >= 1)
                {
                    notation = "cm/s";
                    strNumber = number.ToString("0.00");
                }

                if (number < 1)
                {
                    notation = "mm/s";
                    number *= 100;
                    strNumber = number.ToString("0");
                }
            }

            if (number >= 1000)
            {
                notation = "km/s";
                number /= 1000;
                strNumber = number.ToString("0.000");
            }

            if (negative)
                strNumber = "-" + strNumber;

            return strNumber + notation;
        }

        private string FormatDistance(double number) 
        {
            bool negative = false;
            if (number < 0)
            {
                number = -number;
                negative = true;
            }


            string notation = "m";
            string strNumber = number.ToString("#,##0");

            if (number < 100)
                strNumber = number.ToString("0.000");

            if (number >= 10000)
            {
                notation = "km";
                number /= 1000;
                strNumber = number.ToString("0.000");
            }

            if (number >= 10000)
            {
                notation = "Mm";
                number /= 1000;
                strNumber = number.ToString("0.000");
            }

            if (negative)
                strNumber = "-" + strNumber;

            return strNumber + notation;
        }

        private void InitGUI()
        {
            GUI.skin = HighLogic.Skin;
            winPos = new Rect(UnityEngine.Screen.width - 275, 0, 0, 0);

            if (winStyle == null)
            {
                winStyle = new GUIStyle(GUI.skin.window);
                winStyle.fixedWidth = 275;
                winStyle.stretchHeight = true;
            }

            if (areaStyle == null)
            {
                areaStyle = new GUIStyle(GUI.skin.textArea);
                areaStyle.hover = areaStyle.active = areaStyle.normal;
            }

            if (titleStyle == null)
            {
                titleStyle = new GUIStyle(GUI.skin.label);
                titleStyle.normal.textColor = Color.yellow;
                titleStyle.fontStyle = FontStyle.Bold;
            }

            if (headStyle == null)
            {
                headStyle = new GUIStyle(GUI.skin.label);
                headStyle.normal.textColor = Color.white;
                headStyle.fontStyle = FontStyle.Bold;
            }

            if (dataStyle == null)
            {
                dataStyle = new GUIStyle(GUI.skin.label);
                dataStyle.alignment = TextAnchor.MiddleRight;
            }

            if (buttonStyle == null)
            {
                buttonStyle = new GUIStyle(GUI.skin.button);
            }

            hasInitGUI = true;
        }
    }
}
